<?php  
defined('C5_EXECUTE') or die(_("Access Denied."));

Loader::model('file_attributes'); 

class JbSliderBlockController extends BlockController {
	
	var $pobj;
	
	protected $btTable = 'btJbSlider';
	protected $btInterfaceWidth = "550";
	protected $btInterfaceHeight = "400";
	
	public $defaultDuration = 5;	
	public $defaultFadeDuration = 2;	
	public $images = array();
	public $playback = "ORDER";	
	public $maxThumbs = 0;	

	/** 
	 * Used for localization. If we want to localize the name/description we have to include this
	 */
	public function getBlockTypeDescription() {
		return t("Display a slideshow with sliding overlay.");
	}
	
	public function getBlockTypeName() {
		return t("Slider Overlay");
	}
	
	public function getJavaScriptStrings() {
		return array(
			'choose-file' => t('Choose Image/File'),
			'choose-min-2' => t('Please choose at least two images.'),
			'choose-fileset' => t('Please choose a file set.')
		);
	}
	
	public function on_page_view() {
		$html = Loader::helper('html');
		$bv = new BlockView();
		$bv->setBlockObject($this->getBlockObject());
		//$this->addHeaderItem($html->javascript($bv->getBlockURL().'/js_manual/jquery.ui.widget.js'));
		if($this->enableImageLightbox){
			$this->addHeaderItem($html->javascript($bv->getBlockURL().'/js_manual/jquery.lightbox-0.5.pack.js'));
		}
	}
	
	function prepareImageArray(){
		$gallery_json 		= new stdClass();
		$gallery_json->meta = new stdClass();
		$gallery_json->meta->order = Array();
		$gallery_json->meta->orderPointer = 0;
		$image_info = Array();
		foreach($this->images as $image){
			$tmp = $image;
			$f = File::getByID($image['fID']);
			$tmp['f'] = $f;
			$tmp['caption']	= (array_key_exists('caption', $tmp) && $tmp['caption']) ? $tmp['caption'] : $f->getTitle();					
			$gallery_json->{$tmp['fID']}	 		= new stdClass();			
			$gallery_json->{$tmp['fID']}->src 		= $f->getRelativePath();			
			$gallery_json->{$tmp['fID']}->caption 	= $tmp['caption'];
			$gallery_json->{$tmp['fID']}->titleposition 	= $tmp['titleposition'];
			$gallery_json->meta->order[]=$tmp['fID'];
			$image_info[] = $tmp;
		}
		$jse = Loader::helper('json');
		$this->set('gallery_json', $jse->encode($gallery_json));	
		$this->set('images', $image_info);
	}
	
	function view() {
		$this->loadSliderInformation();
	}

	function add() {
		$this->loadSliderInformation();
	}
	
	function edit() {
		$this->loadSliderInformation();
	}
	

	
	protected function loadSliderInformation() {
		if ($this->fsID == 0) {
			$this->loadImages();
		} else {
			$this->loadFileSet();
		}
		$this->randomizeImages();	
		$this->set('defaultFadeDuration', $this->defaultFadeDuration);
		$this->set('defaultDuration', $this->defaultDuration); 
		$this->set('fadeDuration', $this->fadeDuration);
		$this->set('opacity', $this->opacity);
		$this->set('speed', $this->speed);
		$this->set('imgwidth', $this->imgwidth);
		$this->set('imgheight', $this->imgheight);
		$this->set('colorcode', $this->colorcode);
		$this->set('shownAttributes', $this->shownAttributes);
		$this->set('titleposition', $this->titleposition);
		$this->set('thumbnailWidth', $this->thumbnailWidth);
		$this->set('thumbnailHeight', $this->thumbnailHeight);
		$this->set('thumbnailPerRow', $this->thumbnailPerRow);
		$this->set('maxThumbs', $this->maxThumbs );
		$this->set('enableImageLightbox', $this->enableImageLightbox );
		$this->set('linkToPageField', $this->linkToPageField );
		$this->set('hideoverlay', $this->hideoverlay );
		$this->set('textcolorcode', $this->textcolorcode );
		$this->set('openInNewWindow', $this->openInNewWindow );
		
		
		$this->set('fadeDuration', $this->fadeDuration);
		
		$this->set('duration', $this->duration);
		$this->set('minHeight', $this->minHeight);
		$this->set('fsID', $this->fsID);
		$this->set('fsName', $this->getFileSetName());
		//$this->set('images', $this->images);
		$this->set('playback', $this->playback);
		$type = ($this->fsID > 0) ? 'FILESET' : 'CUSTOM';
		$this->set('type', $type);
		$this->set('bID', $this->bID);			

		$this->prepareImageArray();
	}
	
	function getFileSetName() {
		$db = Loader::db();
		$sql = "SELECT fsName FROM FileSets WHERE fsID=".intval($this->fsID);
		return $db->getOne($sql); 
	}

	public function loadFileSet(){
		$db = Loader::db();
		if (intval($this->fsID) < 1) {
			return false;
		}
		

		if ( class_exists('FileList') ) {
			$fs = FileSet::getByID($this->fsID);
			$fileList = new FileList();		
			$fileList->filterBySet($fs);
			$fileList->filterByType(FileType::T_IMAGE);

			if (method_exists($fileList, 'sortByFileSetDisplayOrder')) {
				$fileList->sortByFileSetDisplayOrder();
			}

			$files = $fileList->get();
		} else {
			$db = Loader::db();
			$sql = "SELECT fsf.fID FROM FileSetFiles fsf WHERE fsf.fsID = " . $this->fsID . ' ORDER BY fsDisplayOrder';
			$fIDs = $db->getCol($sql); 

			$files = array();
			foreach ($fIDs as $fID) {
				$file = File::getByID($fID);
				$files[] = $file;
			}
		}

		$images = array();

		$image = array();
		$image['duration'] = $this->duration;
		$image['fadeDuration'] = $this->fadeDuration;
		$image['groupSet'] = 0;
		$image['url'] = '';
		$maxHeight = 0;
		foreach ($files as $f) {
			$fp = new Permissions($f);
			if(!$fp->canRead()) { continue; }
			$f = $f->getApprovedVersion();
			
			$image['fileName'] = $f->getTitle();
			$image['fID'] = $f->getFileID();
			$image['fullFilePath'] = $f->getRelativePath();
			$image['imgHeight'] = $f->getAttribute("height");
			if ($maxHeight == 0 || $image['imgHeight'] > $maxHeight) {
				$maxHeight = $image['imgHeight'];
			}
			$images[] = $image;
		}

		$this->images = $images;
	}

	function loadImages() {
		$db = Loader::db();
		if(intval($this->bID)==0) $this->images=array();
		$sortChoices=array('ORDER'=>'position','RANDOM-SET'=>'groupSet asc, position asc','RANDOM'=>'rand()');
		if( !array_key_exists($this->playback,$sortChoices) ) 
			$this->playback='ORDER';
		if(intval($this->bID)==0) return array();
		$sql = "SELECT * FROM btJbSliderImg WHERE bID=".intval($this->bID).' ORDER BY '.$sortChoices[$this->playback];
		$this->images=$db->getAll($sql); 
	}
	
	function delete() {
		$db = Loader::db();
		$db->query("DELETE FROM btJbSliderImg WHERE bID=".intval($this->bID));		
		parent::delete();
	}
	
	public function save($data) { 
		$db = Loader::db();
		$args['playback'] = isset($data['playback']) ? trim($data['playback']) : 'ORDER';
		$args['thumbnailWidth'] 	= $data['thumbnailWidth'];
		$args['thumbnailHeight'] 	= $data['thumbnailHeight'];
		$args['thumbnailPerRow'] 	= $data['thumbnailPerRow'];
		$args['imgBackground'] 	= $data['imgBackground'];
		$args['opacity'] 	= $data['opacity'];
		$args['speed'] 	= $data['speed'];
		$args['imgwidth'] 	= $data['imgwidth'];
		$args['colorcode'] = isset($data['colorcode']) ? $data['colorcode'] : '#000000';
		$args['imgheight'] 	= $data['imgheight'];
		$args['maxThumbs'] 	= intval($data['maxThumbs']);
		$args['thumbnailAttributeKeyID'] = 0;
		if ($data['thumbnailAttributeKeyID'] > 0) {
			$args['thumbnailAttributeKeyID'] = $data['thumbnailAttributeKeyID'];
		}
		
		if ($args['thumbnailWidth'] < 1) {
			$args['thumbnailWidth'] = 50;
		}
		if ($args['thumbnailHeight'] < 1) {
			$args['thumbnailHeight'] = 50;
		}
		$args['enableImageLightbox'] = isset($data['enableImageLightbox']) ? $data['enableImageLightbox'] : 0;
		$args['openInNewWindow'] = isset($data['openInNewWindow']) ? 1 : 0;
		$args['linkToPageField'] = isset($data['linkToPageField']) ? trim($data['linkToPageField']) : '';
		$args['hideoverlay'] = isset($data['hideoverlay']) ? $data['hideoverlay'] : 0;
		$args['textcolorcode'] = isset($data['textcolorcode']) ? $data['textcolorcode'] : '#FFFFFF';
		
		
		$allowedFileAttributes=array('title','date','tags','description','filename','extension');
		
		if( $data['type'] == 'FILESET' && $data['fsID'] > 0){
			$args['fsID'] = $data['fsID'];
			$args['duration'] = $data['duration'][0];
			$args['fadeDuration'] = $data['fadeDuration'][0];

			$files = $db->getAll("SELECT fv.fID FROM FileSetFiles fsf, FileVersions fv WHERE fsf.fsID = " . $data['fsID'] .
			         " AND fsf.fID = fv.fID AND fvIsApproved = 1");
			
			//delete existing images
			$db->query("DELETE FROM btJbSliderImg WHERE bID=".intval($this->bID));
			
			//save set displayed attributes			
			$args['shownAttributes'] = '';
			$cleanFileAttrKeys=array();
			$cleanFileProperties=array();  
			if( is_array($data['captionFields']) ){			
				foreach($data['captionFields'] as $propertyName){
					if( in_array($propertyName, $allowedFileAttributes) )
						$cleanFileProperties[]=$propertyName;
					elseif( substr($propertyName,0,4)=='fak_' ) 
						$cleanFileAttrKeys[] = 'fak_'.intval(substr($propertyName,4)); 
				}
			}
			$args['shownAttributes']=join(',', array_merge($cleanFileProperties,$cleanFileAttrKeys) );	
			$args['titleposition'] = $data['titleposition'][0];
		    } else if( $data['type'] == 'CUSTOM' && count($data['imgFIDs']) ){
			$args['fsID'] = 0;
	
			//delete existing images
			$db->query("DELETE FROM btJbSliderImg WHERE bID=".intval($this->bID));
			
			//loop through and add the images
			$pos=0;
			foreach($data['imgFIDs'] as $imgFID){ 			
			
				if(intval($imgFID)==0 || $data['fileNames'][$pos]=='tempFilename') continue;
			
				//save individual image shown attributes 
				$args['shownAttributes'] = '';
				$cleanFileAttrKeys=array();
				$cleanFileProperties=array();  
				if( is_array($data['imgCaptionFieldsFID'.$imgFID]) ){			
					foreach($data['imgCaptionFieldsFID'.$imgFID] as $propertyName){
						if( in_array($propertyName, $allowedFileAttributes) )
							$cleanFileProperties[]=$propertyName;
						elseif( substr($propertyName,0,4)=='fak_' ) 
							$cleanFileAttrKeys[] = 'fak_'.intval(substr($propertyName,4)); 
					}
				}
				$imgShownAttributes=join(',', array_merge($cleanFileProperties,$cleanFileAttrKeys) );	 		
			
				$vals = array(intval($this->bID),intval($imgFID), trim($data['url'][$pos]),intval($data['duration'][$pos]),intval($data['fadeDuration'][$pos]),
						intval($data['groupSet'][$pos]),intval($data['imgHeight'][$pos]),$pos,$data['caption'][$pos],$imgShownAttributes,$data['titleposition'][$pos]);
				$db->query("INSERT INTO btJbSliderImg 
				(bID,fID,url,duration,fadeDuration,groupSet,imgHeight,position,caption,shownAttributes,titleposition) 
				values (?,?,?,?,?,?,?,?,?,?,?)",$vals);
				$pos++;
			}
		}
		
		parent::save($args);
	}
	
	function randomizeImages()
	{
		if($this->playback == 'RANDOM')
		{
			shuffle($this->images);
		}
		else if($this->playback == 'RANDOM-SET')
		{
			$imageGroups=array();
			$imageGroupIds=array();
			$sortedImgs=array();
			foreach($this->images as $imgInfo){
				$imageGroups[$imgInfo['groupSet']][]=$imgInfo;
				if( !in_array($imgInfo['groupSet'],$imageGroupIds) )
					$imageGroupIds[]=$imgInfo['groupSet'];
			}
			shuffle($imageGroupIds);
			foreach($imageGroupIds as $imageGroupId){
				foreach($imageGroups[$imageGroupId] as $imgInfo)
					$sortedImgs[]=$imgInfo;
			}
			$this->images=$sortedImgs;
		}
	}
}

?>