<?php 
/**
 * @package Helpers
 * @category Concrete
 * @author Andrew Embler <andrew@concrete5.org>
 * @copyright  Copyright (c) 2003-2008 Concrete5. (http://www.concrete5.org)
 * @license    http://www.concrete5.org/license/     MIT License
 */

/**
 * Functions useful for working with images.
 * @package Helpers
 * @category Concrete
 * @author Andrew Embler <andrew@concrete5.org>
 * @copyright  Copyright (c) 2003-2008 Concrete5. (http://www.concrete5.org)
 * @license    http://www.concrete5.org/license/     MIT License
 */

defined('C5_EXECUTE') or die(_("Access Denied."));
class ImageresizerSliderHelper
{
	// *** Class variables
	private $image;
	private $width;
	private $height;
	private $imageResized;

	function resize($file,$width,$height,$style,$bgColor = '') {

		if ($file instanceof File) {
			$path = $file->getPath();
		} else {
			$path = $file;
		}

		$ext = pathinfo($path, PATHINFO_EXTENSION);

		// first we check to see if a file with these dimensions exists
		$filename = md5($path . ':' . $width . ':' . $height . ':' . $style . ':' . $bgColor . '-jbslider') . '.' . $ext;

		if (!file_exists(DIR_FILES_CACHE . '/' . $filename)) {
			$this->setFile($path);
			switch ( $style ) {
				case 'auto':
					$this->resizeImage($width, $height, 'auto', $bgColor);
					break;
				case 'fit_both':
					$this->resizeImage($width, $height, 'exact', $bgColor);
					break;
				case 'fit_width':
					$this->resizeImage($width, $height, 'landscape', $bgColor);
					break;
				case 'fit_height':
					$this->resizeImage($width, $height, 'portrait', $bgColor);
					break;
				case 'crop':
					$this->resizeImage($width, $height, 'crop', $bgColor);
					break;
			}
			$this->saveImage(DIR_FILES_CACHE . '/' . $filename, 100);
		}

		$src = REL_DIR_FILES_CACHE . '/' . $filename;
		$abspath = DIR_FILES_CACHE . '/' . $filename;
		$thumb = new stdClass;
		if (isset($abspath) && file_exists($abspath)) {
			$thumb->src = $src;
			$dimensions = getimagesize($abspath);
			$thumb->width = $dimensions[0];
			$thumb->height = $dimensions[1];
			return $thumb;
		}
		return false;
	}

	function setFile($fileName)
	{
		// *** Open up the file
		$this->image = $this->openImage($fileName);

		// *** Get width and height
		$this->width  = imagesx($this->image);
		$this->height = imagesy($this->image);
	}

	## --------------------------------------------------------

	private function openImage($file)
	{
		// *** Get extension
		$extension = strtolower(strrchr($file, '.'));

		switch($extension)
		{
			case '.jpg':
			case '.jpeg':
				$img = @imagecreatefromjpeg($file);
				break;
			case '.gif':
				$img = @imagecreatefromgif($file);
				break;
			case '.png':
				$img = @imagecreatefrompng($file);
				break;
			default:
				$img = false;
				break;
		}

		return $img;
	}

	## --------------------------------------------------------

	public function resizeImage($newWidth, $newHeight, $option="auto", $bgColor = '')
	{
		// *** Get optimal width and height - based on $option
		$optionArray = $this->getDimensions($newWidth, $newHeight, $option);

		$optimalWidth  = $optionArray['optimalWidth'];
		$optimalHeight = $optionArray['optimalHeight'];


		// *** Resample - create image canvas of x, y size
		if ( $option == 'auto' ) {
			$this->imageResized = imagecreatetruecolor($newWidth, $newHeight);
		} else {
			$this->imageResized = imagecreatetruecolor($optimalWidth, $optimalHeight);
		}
		if ( !empty($bgColor) ) {
			$rgb = $this->hex2rgb($bgColor);
			$backgroundColor = imagecolorallocate($this->imageResized, $rgb[0], $rgb[1], $rgb[2]);
			imagefill($this->imageResized, 0, 0, $backgroundColor);
		}
		imagecopyresampled($this->imageResized, $this->image, $optionArray['startX'], $optionArray['startY'], 0, 0, $optimalWidth, $optimalHeight, $this->width, $this->height);


		// *** if option is 'crop', then crop too
		if ($option == 'crop') {
			$this->crop($optimalWidth, $optimalHeight, $newWidth, $newHeight);
		}
	}

	private function hex2rgb($hex) {
		$hex = str_replace("#", "", $hex);

		if(strlen($hex) == 3) {
			$r = hexdec(substr($hex,0,1).substr($hex,0,1));
			$g = hexdec(substr($hex,1,1).substr($hex,1,1));
			$b = hexdec(substr($hex,2,1).substr($hex,2,1));
		} else {
			$r = hexdec(substr($hex,0,2));
			$g = hexdec(substr($hex,2,2));
			$b = hexdec(substr($hex,4,2));
		}
		$rgb = array($r, $g, $b);
		//return implode(",", $rgb); // returns the rgb values separated by commas
		return $rgb; // returns an array with the rgb values
	}

	## --------------------------------------------------------

	private function getDimensions($newWidth, $newHeight, $option)
	{
		$startX = 0;
		$startY = 0;
		switch ($option)
		{

			case 'exact':
				$optimalWidth = $newWidth;
				$optimalHeight= $newHeight;
				break;
			case 'portrait':
				$optimalWidth = $this->getSizeByFixedHeight($newHeight);
				$optimalHeight= $newHeight;
				break;
			case 'landscape':
				$optimalWidth = $newWidth;
				$optimalHeight= $this->getSizeByFixedWidth($newWidth);
				break;
			case 'auto':
				$optionArray = $this->getBestFit($newWidth, $newHeight);
				$optimalWidth = $optionArray['optimalWidth'];
				$optimalHeight = $optionArray['optimalHeight'];
				$startX = $optionArray['offset']['startX'];
				$startY = $optionArray['offset']['startY'];
				break;
			case 'crop':
				$optionArray = $this->getOptimalCrop($newWidth, $newHeight);
				$optimalWidth = $optionArray['optimalWidth'];
				$optimalHeight = $optionArray['optimalHeight'];
				break;
		}
		return array(
			'optimalWidth' => $optimalWidth,
			'optimalHeight' => $optimalHeight,
			'startX' => $startX,
			'startY' => $startY,
		);
	}

	## --------------------------------------------------------

	private function getSizeByFixedHeight($newHeight)
	{
		$ratio = $this->width / $this->height;
		$newWidth = $newHeight * $ratio;
		return $newWidth;
	}

	private function getSizeByFixedWidth($newWidth)
	{
		$ratio = $this->height / $this->width;
		$newHeight = $newWidth * $ratio;
		return $newHeight;
	}

	private function getBestFit($newWidth, $newHeight) {
		$diffWidth = $this->width - $newWidth;
		$diffHeight = $this->height - $newHeight;

		$optimalWidth = $newWidth;
		$optimalHeight = $newHeight;
		$startX = 0;
		$startY = 0;
		$endX = 0;
		$endY = 0;

		if ( $diffWidth < $diffHeight ) { // Fit by height
			$ratio = $this->width / $this->height;
			$optimalWidth = floor($newHeight * $ratio);
			$startX = floor(abs($newWidth - $optimalWidth) / 2);
		} else { // Fit by width
			$ratio = $this->height / $this->width;
			$optimalHeight = floor($newWidth * $ratio);
			$startY = abs(floor($newHeight/2) - floor($optimalHeight/2));
		}

		return array(
			'optimalWidth' => $optimalWidth,
			'optimalHeight' => $optimalHeight,
			'offset' => array(
				'startX' => $startX,
				'startY' => $startY,
				'endX' => $endX,
				'endY' => $endY,
			),
		);
	}

	private function getSizeByAuto($newWidth, $newHeight)
	{
		if ($this->height < $this->width)
			// *** Image to be resized is wider (landscape)
		{
			$optimalWidth = $newWidth;
			$optimalHeight= $this->getSizeByFixedWidth($newWidth);
		}
		elseif ($this->height > $this->width)
			// *** Image to be resized is taller (portrait)
		{
			$optimalWidth = $this->getSizeByFixedHeight($newHeight);
			$optimalHeight= $newHeight;
		}
		else
			// *** Image to be resizerd is a square
		{
			if ($newHeight < $newWidth) {
				$optimalWidth = $newWidth;
				$optimalHeight= $this->getSizeByFixedWidth($newWidth);
			} else if ($newHeight > $newWidth) {
				$optimalWidth = $this->getSizeByFixedHeight($newHeight);
				$optimalHeight= $newHeight;
			} else {
				// *** Sqaure being resized to a square
				$optimalWidth = $newWidth;
				$optimalHeight= $newHeight;
			}
		}

		return array('optimalWidth' => $optimalWidth, 'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function getOptimalCrop($newWidth, $newHeight)
	{

		$heightRatio = $this->height / $newHeight;
		$widthRatio  = $this->width /  $newWidth;

		if ($heightRatio < $widthRatio) {
			$optimalRatio = $heightRatio;
		} else {
			$optimalRatio = $widthRatio;
		}

		$optimalHeight = $this->height / $optimalRatio;
		$optimalWidth  = $this->width  / $optimalRatio;

		return array('optimalWidth' => $optimalWidth, 'optimalHeight' => $optimalHeight);
	}

	## --------------------------------------------------------

	private function crop($optimalWidth, $optimalHeight, $newWidth, $newHeight, $bgColor = '')
	{
		// *** Find center - this will be used for the crop
		$cropStartX = ( $optimalWidth / 2) - ( $newWidth /2 );
		$cropStartY = ( $optimalHeight/ 2) - ( $newHeight/2 );

		$crop = $this->imageResized;
		//imagedestroy($this->imageResized);

		// *** Now crop from center to exact requested size
		$this->imageResized = imagecreatetruecolor($newWidth , $newHeight);
		if ( !empty($bgColor) ) {
			$rgb = $this->hex2rgb($bgColor);
			$backgroundColor = imagecolorallocate($this->imageResized, $rgb[0], $rgb[1], $rgb[2]);
			imagefill($this->imageResized, 0, 0, $backgroundColor);
		}
		imagecopyresampled($this->imageResized, $crop , 0, 0, $cropStartX, $cropStartY, $newWidth, $newHeight , $newWidth, $newHeight);
	}

	## --------------------------------------------------------

	public function saveImage($savePath, $imageQuality="100")
	{
		// *** Get extension
		$extension = strrchr($savePath, '.');
		$extension = strtolower($extension);

		switch($extension)
		{
			case '.jpg':
			case '.jpeg':
				if (imagetypes() & IMG_JPG) {
					$result = imagejpeg($this->imageResized, $savePath, $imageQuality);
				}
				break;

			case '.gif':
				if (imagetypes() & IMG_GIF) {
					imagegif($this->imageResized, $savePath);
				}
				break;

			case '.png':
				// *** Scale quality from 0-100 to 0-9
				$scaleQuality = round(($imageQuality/100) * 9);

				// *** Invert quality setting as 0 is best, not 9
				$invertScaleQuality = 9 - $scaleQuality;

				if (imagetypes() & IMG_PNG) {
					imagepng($this->imageResized, $savePath, $invertScaleQuality);
				}
				break;

			// ... etc

			default:
				// *** No extension - No save.
				break;
		}

		imagedestroy($this->imageResized);
	}


	## --------------------------------------------------------

}